@@ -10,15 +10,14 @@ use Data::Dumper;
 ## no critic (DotMatch,LineBoundary,Sigils,Punctuation,Quotes,Magic,Checked)
 ## no critic (NamingConventions::Capitalization,BracedFileHandle)
 
-our $VERSION = '1.11_02';
+our $VERSION = '1.12';
 our $DEBUG = $ENV{DEBUG} || 0;
 our $XS_LOADED = 0;
 eval { bootstrap Sys::CpuAffinity $VERSION; $XS_LOADED = 1 };
 
 sub TWO () { Math::BigInt->new(2) }
 
-sub import {
-}
+# sub import { }
 
 #
 # Development guide:
@@ -58,9 +57,9 @@ sub getAffinity {
     my $mask = 0
 	|| _getAffinity_with_taskset($pid)
 	|| _getAffinity_with_xs_sched_getaffinity($pid)
-	|| _getAffinity_with_xs_cpuset_getaffinity($pid)
 	|| _getAffinity_with_xs_pthread_self_getaffinity($pid)
 	|| _getAffinity_with_BSD_Process_Affinity($pid)
+	|| _getAffinity_with_xs_freebsd_getaffinity($pid)
 	|| _getAffinity_with_cpuset($pid)
 	|| _getAffinity_with_xs_processor_affinity($pid)
 	|| _getAffinity_with_pbind($pid)
@@ -132,7 +131,7 @@ sub setAffinity {
 	|| _setAffinity_with_taskset($pid,$mask)
 	|| _setAffinity_with_xs_sched_setaffinity($pid,$mask)
 	|| _setAffinity_with_BSD_Process_Affinity($pid,$mask)
-	|| _setAffinity_with_xs_cpuset_setaffinity($pid,$mask) # XXX needs work
+	|| _setAffinity_with_xs_freebsd_setaffinity($pid,$mask)
         || _setAffinity_with_xs_processor_affinity($pid,$mask)
 	|| _setAffinity_with_pbind($pid,$mask)
         || _setAffinity_with_xs_processor_bind($pid,$mask)
@@ -155,7 +154,8 @@ sub getNumCpus {
 	|| _getNumCpus_from_proc_stat()
 	|| _getNumCpus_from_lsdev()
 	|| _getNumCpus_from_bindprocessor()
-	|| _getNumCpus_from_prtconf()   # slower than bindprocessor, lsdev
+	|| _getNumCpus_from_BSD_Process_Affinity()
+	|| _getNumCpus_from_sysctl_freebsd()
 	|| _getNumCpus_from_sysctl()
 	|| _getNumCpus_from_dmesg_bsd()
         || _getNumCpus_from_xs_solaris()
@@ -166,6 +166,7 @@ sub getNumCpus {
 	|| _getNumCpus_from_system_profiler()
 	|| _getNumCpus_from_Win32API_System_Info()
 	|| _getNumCpus_from_Test_Smoke_SysInfo()
+	|| _getNumCpus_from_prtconf()   # slower than bindprocessor, lsdev
 	|| _getNumCpus_from_ENV()
 	|| _getNumCpus_from_taskset()
 	|| -1;
@@ -459,6 +460,11 @@ sub _getNumCpus_from_xs_solaris {
     return $n || 0;
 }
 
+sub _getNumCpus_from_sysctl_freebsd {
+    return 0 unless defined &xs_num_cpus_freebsd;
+    return xs_num_cpus_freebsd() || 0;
+}
+
 sub _getNumCpus_from_dmesg_solaris {
     return 0 if $^O !~ /solaris/i;
     return 0 if !_configExternalProgram('dmesg');
@@ -499,7 +505,7 @@ sub _getNumCpus_from_sysctl {
     return 0 if !_configExternalProgram('sysctl');
     my $cmd = _configExternalProgram('sysctl');
     my @sysctl = qx($cmd -a 2> /dev/null);
-    my @results = grep { /^hw.ncpu\s*[:=]/ } @sysctl;
+    my @results = grep { /^hw.(?:avail|n)cpu\s*[:=]/ } @sysctl;
     _debug("sysctl output:\n@results");
     return 0 if @results == 0;
     my ($ncpus) = $results[0] =~ /[:=]\s*(\d+)/;
@@ -514,6 +520,11 @@ sub _getNumCpus_from_sysctl {
 	_debug("sysctl[3] result: $result");
 	$ncpus = 0 + $result;
     }
+    if ($ncpus == 0) {
+	my $result = qx($cmd -n hw.availcpu 2> /dev/null);
+	_debug("sysctl[4] result: $result");
+	$ncpus = 0 + $result;
+    }
 
 
     return $ncpus || 0;
@@ -864,6 +875,7 @@ sub _getAffinity_with_xs_DEBUG_sched_getaffinity {
     # to debug errors in xs_sched_getaffinity_get_affinity
     # during t/11-exercise-all.t
     my $pid = shift;
+    return 0 if !defined &xs_sched_getaffinity_get_affinity;
     my @mask;
     my $r = xs_sched_getaffinity_get_affinity($pid,\@mask,1);
     if ($r) {
@@ -981,14 +993,12 @@ sub _getAffinity_with_xs_processor_bind {
 sub _getAffinity_with_BSD_Process_Affinity {
     my ($pid) = @_;
     return 0 if $^O !~ /bsd/i;
-    return 0 if !_configModule('BSD::Process::Affinity');
+    return 0 if !_configModule('BSD::Process::Affinity','0.04');
 
     my $mask;
     if (! eval {
-        $mask = BSD::Process::Affinity
-            ->get_process_mask($pid)
-            ->to_bits()->to_Dec();
-        BSD::Process::Affinity->get_process_mask($pid)->get_cpusetid();
+	my $affinity = BSD::Process::Affinity::get_process_mask($pid);
+	$mask = $affinity->get;
         1 }  ) {
         # $MODULE{'BSD::Process::Affinity'} = 0
         _debug("error in _setAffinity_with_BSD_Process_Affinity: $@");
@@ -1016,10 +1026,32 @@ sub _getAffinity_with_cpuset {
     return 0;
 }
 
-sub _getAffinity_with_xs_cpuset_getaffinity {
-    my ($pid) = @_;
-    return 0 if !defined &xs_getaffinity_cpuset_get_affinity;
-    return xs_getaffinity_cpuset_get_affinity($pid);
+sub _getAffinity_with_xs_freebsd_getaffinity {
+    my $pid = shift;
+    return 0 if !defined &xs_getaffinity_freebsd;
+    my @mask = ();
+    my $ret = xs_getaffinity_freebsd($pid,\@mask,0);
+    if ($ret == 0) {
+	return 0;
+    }
+    return _arrayToMask(@mask);
+}
+
+sub _getAffinity_with_xs_freebsd_getaffinity_debug {
+    my $pid = shift;
+    if (!defined &xs_getaffinity_freebsd) {
+        if ($^O =~ /bsd/) {
+            warn "\$^O=$^O, xs_getaffinity_freebsd not defined";
+        }
+        return;
+    }
+    my @mask = ();
+    my $ret = xs_getaffinity_freebsd($pid,\@mask,1);
+    warn "return value from xs_getaffinity_freebsd: $ret";
+    if ($ret == 0) {
+	return 0;
+    }
+    return _arrayToMask(@mask);
 }
 
 sub _getAffinity_with_xs_win32 {
@@ -1232,19 +1264,25 @@ sub _setAffinity_with_xs_sched_setaffinity {
 sub _setAffinity_with_BSD_Process_Affinity {
     my ($pid,$mask) = @_;
     return 0 if $^O !~ /bsd/i;
-    return 0 if !_configModule('BSD::Process::Affinity');
+    return 0 if !_configModule('BSD::Process::Affinity','0.04');
 
     if (not eval {
-        BSD::Process::Affinity
-            ->get_process_mask($pid)
-            ->from_num($mask)
-            ->update();
+	my $affinity = BSD::Process::Affinity::get_process_mask($pid);
+	$affinity->set($mask)->update;
         1}) {
         _debug("error in _setAffinity_with_BSD_Process_Affinity: $@");
         return 0;
     }
 }
 
+sub _getNumCpus_from_BSD_Process_Affinity {
+    return 0 if $^O !~ /bsd/i;
+    return 0 if !_configModule('BSD::Process::Affinity','0.04');
+    my $n = BSD::Process::Affinity::current_set()->get;
+    $n = log( $n+1.01 ) / log(2);
+    return int($n);
+}
+
 sub _setAffinity_with_bindprocessor {
     my ($pid,$mask) = @_;
     return 0 if $^O !~ /aix/i;
@@ -1319,11 +1357,11 @@ sub _setAffinity_with_cpuset {
     return !$c1;
 }
 
-sub _setAffinity_with_xs_cpuset_setaffinity {
+sub _setAffinity_with_xs_freebsd_setaffinity {
     my ($pid,$mask) = @_;
-    # return 0 if $^O !~ /freebsd/i;
-    return 0 if !defined &xs_cpuset_set_affinity;
-    return xs_cpuset_set_affinity($pid,$mask);
+    return 0 if !defined &xs_setaffinity_freebsd;
+    my @mask = _maskToArray($mask);
+    return xs_setaffinity_freebsd($pid,\@mask);
 }
 
 sub _setAffinity_with_xs_win32 {
@@ -1466,13 +1504,20 @@ our %INLINE_CODE = ();
 
 sub _configModule {
     my $module = shift;
+    my $version = shift || "";
     return $MODULE{$module} if defined $MODULE{$module};
 
-    if (eval "require $module") {                 ## no critic (StringyEval)
-        _debug("module $module is available.");
-        return $MODULE{$module} = 1;
+    if (eval "require $module") {        ## no critic (StringyEval)
+	my $v = eval "\$$module" . "::VERSION";
+	if (!$@ && (!$version || $version <= $v)) {
+	    _debug("module $module is available.");
+	    return $MODULE{$module} = 1;
+	} else {
+	    _debug("module $module $version not available ($v)");
+	    return $MODULE{$module} = 0;
+	}
     } else {
-        _debug("module $module not available: $@");
+        _debug("module $module $version not available: $@");
         return $MODULE{$module} = 0;
     }
 }
@@ -1610,7 +1655,7 @@ Sys::CpuAffinity - Set CPU affinity for processes
 
 =head1 VERSION
 
-Version 1.11_02
+Version 1.12
 
 =head1 SYNOPSIS
 
@@ -1852,7 +1897,7 @@ Marty O'Brien, C<< <mob at cpan.org> >>
 
 =head1 LICENSE AND COPYRIGHT
 
-Copyright 2010-2016 Marty O'Brien.
+Copyright 2010-2017 Marty O'Brien.
 
 This program is free software; you can redistribute it and/or modify it
 under the terms of either: the GNU General Public License as published
